@@ -1,11 +1,11 @@
 //! This is an entire parser and interpreter for a dynamically-typed Rust-like expression-oriented
-//! programming language. See `example.nrs` for sample source code.
+//! programming language. See `sample.nrs` for sample source code.
 //! Run it with the following command:
 //! cargo run --example nano_rust -- examples/sample.nrs
 
+use ariadne::{Color, Fmt, Label, Report, ReportKind, Source};
 use chumsky::{prelude::*, stream::Stream};
-use ariadne::{Report, ReportKind, Label, Source, Color, Fmt};
-use std::{collections::HashMap, env, fs, fmt};
+use std::{collections::HashMap, env, fmt, fs};
 
 pub type Span = std::ops::Range<usize>;
 
@@ -59,13 +59,14 @@ fn lexer() -> impl Parser<char, Vec<(Token, Span)>, Error = Simple<char>> {
         .map(Token::Str);
 
     // A parser for operators
-    let op = one_of("+-*/!=".chars())
-        .repeated().at_least(1)
+    let op = one_of("+-*/!=")
+        .repeated()
+        .at_least(1)
         .collect::<String>()
         .map(Token::Op);
 
     // A parser for control characters (delimiters, semicolons, etc.)
-    let ctrl = one_of("()[]{};,".chars()).map(|c| Token::Ctrl(c));
+    let ctrl = one_of("()[]{};,").map(|c| Token::Ctrl(c));
 
     // A parser for identifiers and keywords
     let ident = text::ident().map(|ident: String| match ident.as_str() {
@@ -88,7 +89,10 @@ fn lexer() -> impl Parser<char, Vec<(Token, Span)>, Error = Simple<char>> {
         .or(ident)
         .recover_with(skip_then_retry_until([]));
 
+    let comment = just("//").then(take_until(just('\n'))).padded();
+
     token
+        .padded_by(comment.repeated())
         .map_with_span(|tok, span| (tok, span))
         .padded()
         .repeated()
@@ -124,11 +128,14 @@ impl std::fmt::Display for Value {
             Self::Bool(x) => write!(f, "{}", x),
             Self::Num(x) => write!(f, "{}", x),
             Self::Str(x) => write!(f, "{}", x),
-            Self::List(xs) => write!(f, "[{}]", xs
-                .iter()
-                .map(|x| x.to_string())
-                .collect::<Vec<_>>()
-                .join(", ")),
+            Self::List(xs) => write!(
+                f,
+                "[{}]",
+                xs.iter()
+                    .map(|x| x.to_string())
+                    .collect::<Vec<_>>()
+                    .join(", ")
+            ),
             Self::Func(name) => write!(f, "<function: {}>", name),
         }
     }
@@ -136,9 +143,12 @@ impl std::fmt::Display for Value {
 
 #[derive(Clone, Debug)]
 enum BinaryOp {
-    Add, Sub,
-    Mul, Div,
-    Eq, NotEq,
+    Add,
+    Sub,
+    Mul,
+    Div,
+    Eq,
+    NotEq,
 }
 
 pub type Spanned<T> = (T, Span);
@@ -175,16 +185,17 @@ fn expr_parser() -> impl Parser<Token, Spanned<Expr>, Error = Simple<Token>> + C
                 Token::Str(s) => Ok(Expr::Value(Value::Str(s))),
                 _ => Err(Simple::expected_input_found(span, Vec::new(), Some(tok))),
             })
-                .labelled("value");
+            .labelled("value");
 
             let ident = filter_map(|span, tok| match tok {
                 Token::Ident(ident) => Ok(ident.clone()),
                 _ => Err(Simple::expected_input_found(span, Vec::new(), Some(tok))),
             })
-                .labelled("identifier");
+            .labelled("identifier");
 
             // A list of expressions
-            let items = expr.clone()
+            let items = expr
+                .clone()
                 .chain(just(Token::Ctrl(',')).ignore_then(expr.clone()).repeated())
                 .then_ignore(just(Token::Ctrl(',')).or_not())
                 .or_not()
@@ -199,7 +210,8 @@ fn expr_parser() -> impl Parser<Token, Spanned<Expr>, Error = Simple<Token>> + C
                 .then(expr.clone())
                 .map(|((name, val), body)| Expr::Let(name, Box::new(val), Box::new(body)));
 
-            let list = items.clone()
+            let list = items
+                .clone()
                 .delimited_by(Token::Ctrl('['), Token::Ctrl(']'))
                 .map(Expr::List);
 
@@ -210,14 +222,20 @@ fn expr_parser() -> impl Parser<Token, Spanned<Expr>, Error = Simple<Token>> + C
                 .or(list)
                 // In Nano Rust, `print` is just a keyword, just like Python 2, for simplicity
                 .or(just(Token::Print)
-                    .ignore_then(expr.clone().delimited_by(Token::Ctrl('('), Token::Ctrl(')')))
+                    .ignore_then(
+                        expr.clone()
+                            .delimited_by(Token::Ctrl('('), Token::Ctrl(')')),
+                    )
                     .map(|expr| Expr::Print(Box::new(expr))))
                 .map_with_span(|expr, span| (expr, span))
                 // Atoms can also just be normal expressions, but surrounded with parentheses
-                .or(expr.clone().delimited_by(Token::Ctrl('('), Token::Ctrl(')')))
+                .or(expr
+                    .clone()
+                    .delimited_by(Token::Ctrl('('), Token::Ctrl(')')))
                 // Attempt to recover anything that looks like a parenthesised expression but contains errors
                 .recover_with(nested_delimiters(
-                    Token::Ctrl('('), Token::Ctrl(')'),
+                    Token::Ctrl('('),
+                    Token::Ctrl(')'),
                     [
                         (Token::Ctrl('['), Token::Ctrl(']')),
                         (Token::Ctrl('{'), Token::Ctrl('}')),
@@ -226,7 +244,8 @@ fn expr_parser() -> impl Parser<Token, Spanned<Expr>, Error = Simple<Token>> + C
                 ))
                 // Attempt to recover anything that looks like a list but contains errors
                 .recover_with(nested_delimiters(
-                    Token::Ctrl('['), Token::Ctrl(']'),
+                    Token::Ctrl('['),
+                    Token::Ctrl(']'),
                     [
                         (Token::Ctrl('('), Token::Ctrl(')')),
                         (Token::Ctrl('{'), Token::Ctrl('}')),
@@ -235,34 +254,49 @@ fn expr_parser() -> impl Parser<Token, Spanned<Expr>, Error = Simple<Token>> + C
                 ));
 
             // Function calls have very high precedence so we prioritise them
-            let call = atom.then(items
-                    .delimited_by(Token::Ctrl('('), Token::Ctrl(')'))
-                    .map_with_span(|args, span| (args, span))
-                    .repeated())
+            let call = atom
+                .then(
+                    items
+                        .delimited_by(Token::Ctrl('('), Token::Ctrl(')'))
+                        .map_with_span(|args, span| (args, span))
+                        .repeated(),
+                )
                 .foldl(|f, args| {
                     let span = f.1.start..args.1.end;
                     (Expr::Call(Box::new(f), args), span)
                 });
 
             // Product ops (multiply and divide) have equal precedence
-            let op = just(Token::Op("*".to_string())).to(BinaryOp::Mul).or(just(Token::Op("/".to_string())).to(BinaryOp::Div));
-            let product = call.clone().then(op.then(call).repeated())
+            let op = just(Token::Op("*".to_string()))
+                .to(BinaryOp::Mul)
+                .or(just(Token::Op("/".to_string())).to(BinaryOp::Div));
+            let product = call
+                .clone()
+                .then(op.then(call).repeated())
                 .foldl(|a, (op, b)| {
                     let span = a.1.start..b.1.end;
                     (Expr::Binary(Box::new(a), op, Box::new(b)), span)
                 });
 
             // Sum ops (add and subtract) have equal precedence
-            let op = just(Token::Op("+".to_string())).to(BinaryOp::Add).or(just(Token::Op("-".to_string())).to(BinaryOp::Sub));
-            let sum = product.clone().then(op.then(product).repeated())
+            let op = just(Token::Op("+".to_string()))
+                .to(BinaryOp::Add)
+                .or(just(Token::Op("-".to_string())).to(BinaryOp::Sub));
+            let sum = product
+                .clone()
+                .then(op.then(product).repeated())
                 .foldl(|a, (op, b)| {
                     let span = a.1.start..b.1.end;
                     (Expr::Binary(Box::new(a), op, Box::new(b)), span)
                 });
 
             // Comparison ops (equal, not-equal) have equal precedence
-            let op = just(Token::Op("==".to_string())).to(BinaryOp::Eq).or(just(Token::Op("!=".to_string())).to(BinaryOp::NotEq));
-            let compare = sum.clone().then(op.then(sum).repeated())
+            let op = just(Token::Op("==".to_string()))
+                .to(BinaryOp::Eq)
+                .or(just(Token::Op("!=".to_string())).to(BinaryOp::NotEq));
+            let compare = sum
+                .clone()
+                .then(op.then(sum).repeated())
                 .foldl(|a, (op, b)| {
                     let span = a.1.start..b.1.end;
                     (Expr::Binary(Box::new(a), op, Box::new(b)), span)
@@ -272,11 +306,13 @@ fn expr_parser() -> impl Parser<Token, Spanned<Expr>, Error = Simple<Token>> + C
         });
 
         // Blocks are expressions but delimited with braces
-        let block = expr.clone()
+        let block = expr
+            .clone()
             .delimited_by(Token::Ctrl('{'), Token::Ctrl('}'))
             // Attempt to recover anything that looks like a block but contains errors
             .recover_with(nested_delimiters(
-                Token::Ctrl('{'), Token::Ctrl('}'),
+                Token::Ctrl('{'),
+                Token::Ctrl('}'),
                 [
                     (Token::Ctrl('('), Token::Ctrl(')')),
                     (Token::Ctrl('['), Token::Ctrl(']')),
@@ -288,23 +324,33 @@ fn expr_parser() -> impl Parser<Token, Spanned<Expr>, Error = Simple<Token>> + C
             just(Token::If)
                 .ignore_then(expr.clone())
                 .then(block.clone())
-                .then(just(Token::Else).ignore_then(block.clone().or(if_)).or_not())
-                .map_with_span(|((cond, a), b), span| (
-                    Expr::If(Box::new(cond), Box::new(a), Box::new(match b {
-                        Some(b) => b,
-                        // If an `if` expression has no trailing `else` block, we magic up one that just produces null
-                        None => (Expr::Value(Value::Null), span.clone()),
-                    })),
-                    span,
-                ))
+                .then(
+                    just(Token::Else)
+                        .ignore_then(block.clone().or(if_))
+                        .or_not(),
+                )
+                .map_with_span(|((cond, a), b), span| {
+                    (
+                        Expr::If(
+                            Box::new(cond),
+                            Box::new(a),
+                            Box::new(match b {
+                                Some(b) => b,
+                                // If an `if` expression has no trailing `else` block, we magic up one that just produces null
+                                None => (Expr::Value(Value::Null), span.clone()),
+                            }),
+                        ),
+                        span,
+                    )
+                })
         });
 
         // Both blocks and `if` are 'block expressions' and can appear in the place of statements
-        let block_expr = block
-            .or(if_)
-            .labelled("block");
+        let block_expr = block.or(if_).labelled("block");
 
-        let block_chain = block_expr.clone().then(block_expr.clone().repeated())
+        let block_chain = block_expr
+            .clone()
+            .then(block_expr.clone().repeated())
             .foldl(|a, b| {
                 let span = a.1.start..b.1.end;
                 (Expr::Then(Box::new(a), Box::new(b)), span)
@@ -317,10 +363,13 @@ fn expr_parser() -> impl Parser<Token, Spanned<Expr>, Error = Simple<Token>> + C
             .foldl(|a, b| {
                 let span = a.1.clone(); // TODO: Not correct
                 (
-                    Expr::Then(Box::new(a), Box::new(match b {
-                        Some(b) => b,
-                        None => (Expr::Value(Value::Null), span.clone()),
-                    })),
+                    Expr::Then(
+                        Box::new(a),
+                        Box::new(match b {
+                            Some(b) => b,
+                            None => (Expr::Value(Value::Null), span.clone()),
+                        }),
+                    ),
                     span,
                 )
             })
@@ -334,38 +383,46 @@ fn funcs_parser() -> impl Parser<Token, HashMap<String, Func>, Error = Simple<To
     });
 
     // Argument lists are just identifiers separated by commas, surrounded by parentheses
-    let args = ident.clone()
+    let args = ident
+        .clone()
         .separated_by(just(Token::Ctrl(',')))
         .allow_trailing()
         .delimited_by(Token::Ctrl('('), Token::Ctrl(')'))
         .labelled("function args");
 
     let func = just(Token::Fn)
-        .ignore_then(ident
-            .map_with_span(|name, span| (name, span))
-            .labelled("function name"))
+        .ignore_then(
+            ident
+                .map_with_span(|name, span| (name, span))
+                .labelled("function name"),
+        )
         .then(args)
-        .then(expr_parser()
-            .delimited_by(Token::Ctrl('{'), Token::Ctrl('}'))
-            // Attempt to recover anything that looks like a function body but contains errors
-            .recover_with(nested_delimiters(
-                Token::Ctrl('{'), Token::Ctrl('}'),
-                [
-                    (Token::Ctrl('('), Token::Ctrl(')')),
-                    (Token::Ctrl('['), Token::Ctrl(']')),
-                ],
-                |span| (Expr::Error, span),
-            )))
+        .then(
+            expr_parser()
+                .delimited_by(Token::Ctrl('{'), Token::Ctrl('}'))
+                // Attempt to recover anything that looks like a function body but contains errors
+                .recover_with(nested_delimiters(
+                    Token::Ctrl('{'),
+                    Token::Ctrl('}'),
+                    [
+                        (Token::Ctrl('('), Token::Ctrl(')')),
+                        (Token::Ctrl('['), Token::Ctrl(']')),
+                    ],
+                    |span| (Expr::Error, span),
+                )),
+        )
         .map(|((name, args), body)| (name, Func { args, body }))
         .labelled("function");
 
-    func
-        .repeated()
+    func.repeated()
         .try_map(|fs, _| {
             let mut funcs = HashMap::new();
             for ((name, name_span), f) in fs {
                 if funcs.insert(name.clone(), f).is_some() {
-                    return Err(Simple::custom(name_span.clone(), format!("Function '{}' already exists", name)));
+                    return Err(Simple::custom(
+                        name_span.clone(),
+                        format!("Function '{}' already exists", name),
+                    ));
                 }
             }
             Ok(funcs)
@@ -378,14 +435,20 @@ struct Error {
     msg: String,
 }
 
-fn eval_expr(expr: &Spanned<Expr>, funcs: &HashMap<String, Func>, stack: &mut Vec<(String, Value)>) -> Result<Value, Error> {
+fn eval_expr(
+    expr: &Spanned<Expr>,
+    funcs: &HashMap<String, Func>,
+    stack: &mut Vec<(String, Value)>,
+) -> Result<Value, Error> {
     Ok(match &expr.0 {
         Expr::Error => unreachable!(), // Error expressions only get created by parser errors, so cannot exist in a valid AST
         Expr::Value(val) => val.clone(),
-        Expr::List(items) => Value::List(items
-            .iter()
-            .map(|item| eval_expr(item, funcs, stack))
-            .collect::<Result<_, _>>()?),
+        Expr::List(items) => Value::List(
+            items
+                .iter()
+                .map(|item| eval_expr(item, funcs, stack))
+                .collect::<Result<_, _>>()?,
+        ),
         Expr::Local(name) => stack
             .iter()
             .rev()
@@ -402,17 +465,33 @@ fn eval_expr(expr: &Spanned<Expr>, funcs: &HashMap<String, Func>, stack: &mut Ve
             let res = eval_expr(body, funcs, stack)?;
             stack.pop();
             res
-        },
+        }
         Expr::Then(a, b) => {
             eval_expr(a, funcs, stack)?;
             eval_expr(b, funcs, stack)?
-        },
-        Expr::Binary(a, BinaryOp::Add, b) => Value::Num(eval_expr(a, funcs, stack)?.num(a.1.clone())? + eval_expr(b, funcs, stack)?.num(b.1.clone())?),
-        Expr::Binary(a, BinaryOp::Sub, b) => Value::Num(eval_expr(a, funcs, stack)?.num(a.1.clone())? - eval_expr(b, funcs, stack)?.num(b.1.clone())?),
-        Expr::Binary(a, BinaryOp::Mul, b) => Value::Num(eval_expr(a, funcs, stack)?.num(a.1.clone())? * eval_expr(b, funcs, stack)?.num(b.1.clone())?),
-        Expr::Binary(a, BinaryOp::Div, b) => Value::Num(eval_expr(a, funcs, stack)?.num(a.1.clone())? / eval_expr(b, funcs, stack)?.num(b.1.clone())?),
-        Expr::Binary(a, BinaryOp::Eq, b) => Value::Bool(eval_expr(a, funcs, stack)? == eval_expr(b, funcs, stack)?),
-        Expr::Binary(a, BinaryOp::NotEq, b) => Value::Bool(eval_expr(a, funcs, stack)? != eval_expr(b, funcs, stack)?),
+        }
+        Expr::Binary(a, BinaryOp::Add, b) => Value::Num(
+            eval_expr(a, funcs, stack)?.num(a.1.clone())?
+                + eval_expr(b, funcs, stack)?.num(b.1.clone())?,
+        ),
+        Expr::Binary(a, BinaryOp::Sub, b) => Value::Num(
+            eval_expr(a, funcs, stack)?.num(a.1.clone())?
+                - eval_expr(b, funcs, stack)?.num(b.1.clone())?,
+        ),
+        Expr::Binary(a, BinaryOp::Mul, b) => Value::Num(
+            eval_expr(a, funcs, stack)?.num(a.1.clone())?
+                * eval_expr(b, funcs, stack)?.num(b.1.clone())?,
+        ),
+        Expr::Binary(a, BinaryOp::Div, b) => Value::Num(
+            eval_expr(a, funcs, stack)?.num(a.1.clone())?
+                / eval_expr(b, funcs, stack)?.num(b.1.clone())?,
+        ),
+        Expr::Binary(a, BinaryOp::Eq, b) => {
+            Value::Bool(eval_expr(a, funcs, stack)? == eval_expr(b, funcs, stack)?)
+        }
+        Expr::Binary(a, BinaryOp::NotEq, b) => {
+            Value::Bool(eval_expr(a, funcs, stack)? != eval_expr(b, funcs, stack)?)
+        }
         Expr::Call(func, (args, args_span)) => {
             let f = eval_expr(func, funcs, stack)?;
             match f {
@@ -431,41 +510,47 @@ fn eval_expr(expr: &Spanned<Expr>, funcs: &HashMap<String, Func>, stack: &mut Ve
                             .collect::<Result<_, _>>()?
                     };
                     eval_expr(&f.body, funcs, &mut stack)?
-                },
-                f => return Err(Error {
-                    span: func.1.clone(),
-                    msg: format!("'{:?}' is not callable", f),
-                }),
+                }
+                f => {
+                    return Err(Error {
+                        span: func.1.clone(),
+                        msg: format!("'{:?}' is not callable", f),
+                    })
+                }
             }
-        },
+        }
         Expr::If(cond, a, b) => {
             let c = eval_expr(cond, funcs, stack)?;
             match c {
                 Value::Bool(true) => eval_expr(a, funcs, stack)?,
                 Value::Bool(false) => eval_expr(b, funcs, stack)?,
-                c => return Err(Error {
-                    span: cond.1.clone(),
-                    msg: format!("Conditions must be booleans, found '{:?}'", c),
-                }),
+                c => {
+                    return Err(Error {
+                        span: cond.1.clone(),
+                        msg: format!("Conditions must be booleans, found '{:?}'", c),
+                    })
+                }
             }
-        },
+        }
         Expr::Print(a) => {
             let val = eval_expr(a, funcs, stack)?;
             println!("{}", val);
             val
-        },
+        }
     })
 }
 
 fn main() {
-    let src = fs::read_to_string(env::args().nth(1).expect("Expected file argument")).expect("Failed to read file");
+    let src = fs::read_to_string(env::args().nth(1).expect("Expected file argument"))
+        .expect("Failed to read file");
 
     let (tokens, mut errs) = lexer().parse_recovery(src.as_str());
 
     let parse_errs = if let Some(tokens) = tokens {
         // println!("Tokens = {:?}", tokens);
         let len = src.chars().count();
-        let (ast, parse_errs) = funcs_parser().parse_recovery(Stream::from_iter(len..len + 1, tokens.into_iter()));
+        let (ast, parse_errs) =
+            funcs_parser().parse_recovery(Stream::from_iter(len..len + 1, tokens.into_iter()));
 
         println!("{:#?}", ast);
         if let Some(funcs) = ast.filter(|_| errs.len() + parse_errs.len() == 0) {
@@ -485,47 +570,73 @@ fn main() {
         Vec::new()
     };
 
-    errs
-        .into_iter()
+    errs.into_iter()
         .map(|e| e.map(|c| c.to_string()))
-        .chain(parse_errs
-            .into_iter()
-            .map(|e| e.map(|tok| tok.to_string())))
+        .chain(parse_errs.into_iter().map(|e| e.map(|tok| tok.to_string())))
         .for_each(|e| {
             let report = Report::build(ReportKind::Error, (), e.span().start);
 
             let report = match e.reason() {
                 chumsky::error::SimpleReason::Unclosed { span, delimiter } => report
-                    .with_message(format!("Unclosed delimiter {}", delimiter.fg(Color::Yellow)))
-                    .with_label(Label::new(span.clone())
-                        .with_message(format!("Unclosed delimiter {}", delimiter.fg(Color::Yellow)))
-                    .with_color(Color::Yellow))
-                    .with_label(Label::new(e.span())
-                        .with_message(format!("Must be closed before this {}", e.found().unwrap_or(&"end of file".to_string()).fg(Color::Red)))
-                    .with_color(Color::Red)),
+                    .with_message(format!(
+                        "Unclosed delimiter {}",
+                        delimiter.fg(Color::Yellow)
+                    ))
+                    .with_label(
+                        Label::new(span.clone())
+                            .with_message(format!(
+                                "Unclosed delimiter {}",
+                                delimiter.fg(Color::Yellow)
+                            ))
+                            .with_color(Color::Yellow),
+                    )
+                    .with_label(
+                        Label::new(e.span())
+                            .with_message(format!(
+                                "Must be closed before this {}",
+                                e.found()
+                                    .unwrap_or(&"end of file".to_string())
+                                    .fg(Color::Red)
+                            ))
+                            .with_color(Color::Red),
+                    ),
                 chumsky::error::SimpleReason::Unexpected => report
-                    .with_message(format!("{}, expected {}", if e.found().is_some() {
-                        "Unexpected token in input"
-                    } else {
-                        "Unexpected end of input"
-                    }, if e.expected().len() == 0 {
-                        "end of input".to_string()
-                    } else {
-                        e.expected().map(|x| x.to_string()).collect::<Vec<_>>().join(", ")
-                    }))
-                    .with_label(Label::new(e.span())
-                        .with_message(format!("Unexpected token {}", e.found().unwrap_or(&"end of file".to_string()).fg(Color::Red)))
-                    .with_color(Color::Red)),
-                chumsky::error::SimpleReason::Custom(msg) => report
-                    .with_message(msg)
-                    .with_label(Label::new(e.span())
+                    .with_message(format!(
+                        "{}, expected {}",
+                        if e.found().is_some() {
+                            "Unexpected token in input"
+                        } else {
+                            "Unexpected end of input"
+                        },
+                        if e.expected().len() == 0 {
+                            "something else".to_string()
+                        } else {
+                            e.expected()
+                                .map(|expected| match expected {
+                                    Some(expected) => expected.to_string(),
+                                    None => "end of input".to_string(),
+                                })
+                                .collect::<Vec<_>>()
+                                .join(", ")
+                        }
+                    ))
+                    .with_label(
+                        Label::new(e.span())
+                            .with_message(format!(
+                                "Unexpected token {}",
+                                e.found()
+                                    .unwrap_or(&"end of file".to_string())
+                                    .fg(Color::Red)
+                            ))
+                            .with_color(Color::Red),
+                    ),
+                chumsky::error::SimpleReason::Custom(msg) => report.with_message(msg).with_label(
+                    Label::new(e.span())
                         .with_message(format!("{}", msg.fg(Color::Red)))
-                    .with_color(Color::Red)),
+                        .with_color(Color::Red),
+                ),
             };
 
-            report
-                .finish()
-                .print(Source::from(&src))
-                .unwrap();
+            report.finish().print(Source::from(&src)).unwrap();
         });
 }
