@@ -3,7 +3,7 @@ use std::ops::Range;
 /// A trait that describes a span over a particular range of inputs.
 ///
 /// Spans typically consist of some context, such as the file they originated from, and a start/end offset. Spans are
-/// permitted to overlap one-another. The end offset must always be greater than or equal to the end offset.
+/// permitted to overlap one-another. The end offset must always be greater than or equal to the start offset.
 ///
 /// Span is automatically implemented for [`Range<T>`] and [`(C, Range<T>)`].
 pub trait Span: Clone {
@@ -43,18 +43,32 @@ impl<T: Clone + Ord> Span for Range<T> {
     type Context = ();
     type Offset = T;
 
-    fn new((): Self::Context, range: Self) -> Self { range }
+    fn new((): Self::Context, range: Self) -> Self {
+        range
+    }
     fn context(&self) -> Self::Context {}
-    fn start(&self) -> Self::Offset { self.start.clone() }
-    fn end(&self) -> Self::Offset { self.end.clone() }
+    fn start(&self) -> Self::Offset {
+        self.start.clone()
+    }
+    fn end(&self) -> Self::Offset {
+        self.end.clone()
+    }
 }
 
 impl<C: Clone, T: Clone> Span for (C, Range<T>) {
     type Context = C;
     type Offset = T;
 
-    fn new(context: Self::Context, range: Range<T>) -> Self { (context, range) }
-    fn context(&self) -> Self::Context { self.0.clone() }
-    fn start(&self) -> Self::Offset { self.1.start.clone() }
-    fn end(&self) -> Self::Offset { self.1.end.clone() }
+    fn new(context: Self::Context, range: Range<T>) -> Self {
+        (context, range)
+    }
+    fn context(&self) -> Self::Context {
+        self.0.clone()
+    }
+    fn start(&self) -> Self::Offset {
+        self.1.start.clone()
+    }
+    fn end(&self) -> Self::Offset {
+        self.1.end.clone()
+    }
 }
